/*240x320 TouchScreen Alignment Tool
   2022 - Gord Payne
   Published on Instructables.com under 'gpteacher'
*/
int voltage = 9;// either 5V(USB) or 9V(battery)
int px, py;// screen display (x,y) coordinates

// x mapping coefficients 5Volt USB and 9V battery
// coefficients derived from Desmos.com Quadratic Regression Analysis
// Mapping x - coefficients   '5' means 5 volt, '9' means 9 volt
double x5a = -0.0000137405;
double x5b = -0.407394;
double x5c = 390.651;
double x9a = 0.00000196662;
double x9b = -0.440977;
double x9c = 415.525;

// Mapping y- coefficients
double y5a = 0.0000235224;
double y5b = -0.344986;
double y5c = 292.274;
double y9a = 0.00000388783;
double y9b = -0.343101;
double y9c = 307.304;
double aXv, bXv, cXv, aYv, bYv, cYv;

const int alignC   = 9;// number of (x,y) points on the display
const int aX[] = {20, 160, 300, 20, 160, 300, 20, 160, 300}; // align (x,y) grid references
const int aY[]  = {20, 20, 20, 120, 120, 120, 220, 220, 220};
#define MINPRESSURE  10
#define MAXPRESSURE  1000
#include <Adafruit_GFX.h> // graphics library

#include <MCUFRIEND_kbv.h> // screen control library
MCUFRIEND_kbv tft;

#define LCD_CS A3 // Chip Select goes to Analog 3
#define LCD_CD A2 // Command/Data goes to Analog 2
#define LCD_WR A1 // LCD Write goes to Analog 1
#define LCD_RD A0 // LCD Read goes to Analog 0
#define LCD_RESET A4 // Can alternately just connect to Arduino's reset pin


#define YP A1   //A3 for ILI9320
#define YM 7
#define XM A2
#define XP 6

#include <TouchScreen.h>
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);
TSPoint tp;
// Assign human-readable names to some common 16-bit color values:
#define  BLACK   0x0000
#define RED     0xF800
#define WHITE   0xFFFF

int offSet = 0; // the offset for the dispay point index numbers so they all are on screen
long lastTouch = millis();
long tThresh = 120;

int tMode = 0;//0 = draw grid, 1 = sense touch

void setup()
{
  Serial.begin(9600);
  digitalWrite(A0, HIGH);
  pinMode(A0, OUTPUT);
  uint16_t ID = tft.readID(); //need these two ID lines!!!
  tft.begin(ID);
  tft.setRotation(1);// LANDSCAPE MODE = 1, Portrait mode = 0

  setCequations();// set coordinate constants based on voltage setting.
}

void loop() {
  if (tMode == 0) {// switch between display and touch modes
    alignGrid();// draw the alignment grid
    tMode = 1;// switch to touch detection mode
  } else {
    alignTouch();// wait to detect a screen touch
  }

}

///////////////////////////


void alignGrid() {
  pinMode(YP, OUTPUT);  //restore the TFT control pins
  pinMode(XM, OUTPUT);// for display after detecting a touch
  tft.setTextColor(BLACK);
  tft.fillScreen(WHITE);
  tft.setFont();// back to standard font
  tft.setTextSize(2);
  for (int i = 0; i < alignC; i++) {// draw black circles with white centres
    tft.fillCircle(aX[i], aY[i], 10, BLACK);
    tft.fillCircle(aX[i], aY[i], 1, WHITE);
    int offSet = 10;
    if (aY[i] > 180) offSet = -35;// for low part of screen, shift the numbers up so they
    // are visible

    tft.setCursor(aX[i] , aY[i] + 5 + offSet);
    tft.print(i);
  }

  // touch areas for switching between 5 Volt and 9 Volt mapping coefficients
  tft.fillRect(80, 90, 50, 50, BLACK);
  tft.fillRect(190, 90, 50, 50, RED);
  tft.setCursor(90, 100);
  tft.setTextColor(WHITE);
  tft.print("5V");
  tft.setCursor(200, 100);
  tft.print("9V");
}
void alignTouch() {// detect a screen touch
  if (millis() - lastTouch > tThresh) {
    TSPoint p = ts.getPoint();
    if ((p.z > MINPRESSURE && p.z < MAXPRESSURE)) {
      map2d(p.y, p.x);// touch coordinates to screen display coordinates
      if ((px > 75) && (px < 120) && (py > 90) && (py < 140)) {// switch to 5 Volt
        Serial.println("5V");
        pinMode(YP, OUTPUT);  //restore the TFT control pins
        pinMode(XM, OUTPUT);// for display after detecting a touch
        tft.fillRect(50, 20, 40, 20, WHITE);
        tft.setCursor(50, 20);
        tft.setTextColor(BLACK);
        tft.print("5V");
        voltage = 5;
        setCequations();// set coefficients for 5 Volt mappings

      } else if ((px > 203) && (px < 220) && (py > 85) && (py < 138)) {// switch to 9 Volt
        pinMode(YP, OUTPUT);  //restore the TFT control pins
        pinMode(XM, OUTPUT);
        tft.fillRect(50, 20, 40, 20, WHITE);
        tft.setCursor(50, 20);
        tft.setTextColor(BLACK);
        Serial.println("9V");
        pinMode(YP, OUTPUT);  //restore the TFT control pins
        pinMode(XM, OUTPUT);// for display after detecting a touch tft.setCursor(50,20);
        tft.setTextColor(BLACK);
        tft.print("9V");
        voltage = 9;
        setCequations();//set coefficients for mappings when using 9 Volt supply

      }
      showAlign(p);// draw a red circle at the touch point
    }

    lastTouch = millis();// reset for next touch detection

  }

}

void showAlign(TSPoint p) {
  pinMode(YP, OUTPUT);  //restore the TFT control pins
  pinMode(XM, OUTPUT);// for display after detecting a touch
  tft.setTextColor(BLACK);
  map2d(p.y, p.x);// map the touch coordinates to screen display coordinates
  // notice that with Landscape orientation, the touch coordinate are actually (y,x)


  tft.fillRect(50, 70, 120, 20, WHITE);
  tft.setCursor(50, 70);
  tft.print(p.y);
  tft.print(", ");
  tft.print(p.x);
  //px = map(p.y, 930, 180, 0, 320);// basic Arduino map method. Not useful
  //py = map(p.x, 900, 180, 0, 240);
  tft.setCursor(50, 180);
  tft.fillRect(50, 180, 180, 25, WHITE);
  // display the mapped screen display coordinates
  tft.print(px);
  tft.print(", ");
  tft.print(py);

  tft.fillCircle(px, py, 10, RED);// draw red circle at the mapped coordinates
}

void setCequations() {// set the quadratic regression coefficients based on voltage choice
  // Best mappings determined to be quadratic y = ax^2+bx+c
  if (voltage == 5) {
    aXv = x5a;
    bXv = x5b;
    cXv = x5c;
    aYv = y5a;
    bYv = y5b;
    cYv = y5c;
  } else {
    aXv = x9a;
    bXv = x9b;
    cXv = x9c;
    aYv = y9a;
    bYv = y9b;
    cYv = y9c;
  }
}

void map2d(int x, int y) {// map incoming touch coordinates to screen coordinates
  // convert touch cooridinates to screen coordinates
  // output is based on quadratic regression, so using quadratic functions
  double cfX = aXv * pow(x, 2) + bXv * x + cXv;
  double cfY = aYv * pow(y, 2) + bYv * y + cYv;
  px = (int)round(cfX);// round the decimal outputs to integers for screen display
  py = (int)round(cfY);
}
